module.exports = function(grunt)
{
	grunt.loadNpmTasks('grunt-contrib-clean');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-connect');
	grunt.loadNpmTasks('grunt-contrib-copy');
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-jsdoc');
	grunt.loadNpmTasks('grunt-open');
	grunt.loadTasks('./tasks');

	grunt.initConfig({
		// The working directories
		compile_dir : 'build',
		dist_dir : 'dist',
		source_dir : 'src',
		docs_dir : 'docs',

		// All the source files
		src : {
			gamename : [
				'src/Loader.js'
			]
		},
		pkg : grunt.file.readJSON('package.json'),

		// Compile stuff
		clean : {
			build : {
				src : [ '<%= compile_dir %>' ]
			},
			jsdoc : {
				src : [ '<%= docs_dir %>' ]
			},
		},
		concat : {
			gamename : {
				options : {
					process : {
						data : {
							version : '<%= pkg.version %>',
							buildDate : '<%= grunt.template.today() %>'
						}
					}
				},
				src : [ '<%= src.gamename %>' ],
				dest : '<%= compile_dir %>/gamename.js'
			}
		},
		umd : {
			gamename : {
				src : '<%= concat.gamename.dest %>',
				dest : '<%= umd.gamename.src %>'
			}
		},
		uglify : {
			gamename : {
				options : {
					banner : '/*! Game Name v<%= pkg.version %> | (c) 2013 Company. */\n'
				},
				src : [ '<%= umd.gamename.dest %>' ],
				dest : '<%= compile_dir %>/gamename.min.js'
			}
		},
		copy : {
			gamename : {
				nonull: true,
				src: '<%= compile_dir %>/gamename.min.js',
				dest: '<%= dist_dir %>/gamename.js',
			}
		},
		
		// Documentation stuff
		jsdoc : {
			dist : {
				src: ['readme.md',
					'<%= source_dir %>',
					],
				options: {
					destination: '<%= docs_dir%>',
					lenient : true // Continue even if there are any errors
				}
			}
		},

		// Testing stuff
		connect: {
			server: {
				options: {
					port: 8080,
					base: ['test_files', 'build']
				}
			}
		},
		watch: {
			files: 'src/**/*.js',
			tasks: ['concat', 'umd']
		},
		open: {
			dev: {
				path: 'http://localhost:8080/index.html'
			}
		},

		// Check the code
		jshint: {
			all: ['Gruntfile.js', '<%= source_dir %>/**/*.js', '<%= compile_dir %>/gamename.js']
		}
	});

	grunt.registerTask('default', ['connect', 'open', 'watch']);
	grunt.registerTask('build', [ 'clean', 'concat', 'uglify', 'copy' ]);
	grunt.registerTask('check', [ 'jshint' ]);
	grunt.registerTask('docs', [ 'clean', 'jsdoc' ]);
};
